/* ==========================================================================*/
/* - - - - - - - - - -   B o o t L o a d 2 3 2 . C   - - - - - - - - - - - - */
/* ==========================================================================*/
/*	Created   12-27-2003   M. Wheatley                                       */
/*	Modified Mar 22, 2003  MW                                                */
/*.........................................................................  */
/* Used by VersaBeacon porject                                               */
/*---------------------------------------------------------------------------*/
/*------------------------> I N C L U D E S <--------------------------------*/
/*---------------------------------------------------------------------------*/
#include <16F876.h>
#NOLIST
//ASCP link protocol definitions
#include <C:\Documents and Settings\Administrator\My Documents\ASCP\ascpdefs.h>
//#include <ascpdefs.h>
#LIST

#case		// Make the compiler case sensitive
/*---------------------------------------------------------------------------*/
/*----------------> P R O C E S S O R    S P E C I F I C <-------------------*/
/*---------------------------------------------------------------------------*/
#define CLOCK_FREQ 6000000	// PIC Oscillator Frequency

//NOTE!!! New version compiler 3.182 decided to define 
// Program EEROM Write Enable as NOWRT!!  go figure.
//   Beware it may change back someday.
#fuses HS,PUT,NOBROWNOUT,WDT,NOPROTECT,NOLVP,NOWRT,NOCPD,NODEBUG
#use delay(clock=CLOCK_FREQ)
#use rs232 (baud=19200, xmit=PIN_C6, rcv=PIN_C7)  // Use the hardware USART
#use fast_io(A)
#use fast_io(B)
#use fast_io(C)

/*    Define the input/output port pins for the 877 based interface          */
#define SKIPBOOT PIN_B7 	/* Read to enter Boot loader or not              */

#define PORTA_DIR 0x1C	/* port A direction TRIS bits   (1==INPUT)           */
#define PORTB_DIR 0xCF	/* port B direction TRIS bits                        */
#define PORTC_DIR 0x90	/* port C direction TRIS bits                        */

#BIT TMR1IF = 0x0C.0

#byte SSPBUF  = 0x13	//define the serial port registers
#byte SSPCON  = 0x14	// since the built in functions aren't used
#byte SSPCON2 = 0x91
#byte SSPADD = 0x93
#byte SSPSTAT  = 0x94
#byte TXSTA  = 0x98
#byte TXREG  = 0x19
#byte RCSTA  = 0x18
#byte RCREG  = 0x1A
#byte SPBRG  = 0x99

#byte PIR1  = 0x0C
#byte INTCON  = 0x0B
/*---------------------------------------------------------------------------*/
/*--------------------> L O C A L   D E F I N E S <--------------------------*/
/*---------------------------------------------------------------------------*/
#define TIMER_TICS_PER_SECOND (CLOCK_FREQ/(4*256*256))
#define MSG_TIMEOUT_TIME (TIMER_TICS_PER_SECOND*2)

#define MAX_MSG ((32*2)+6)		//32 words plus header

#define BOOTFLAG0 0x5A	//when reset, if location 0x7E,0x7F equal 0x5AA5
#define BOOTFLAG1 0xA5	//then it means that it should jump into the programming
						// mode ignoring the switch and checksum settings.

#define BOOTSECURE0 0x5A	// The next 4 bytes are security codes for entering
#define BOOTSECURE1 0xFF	// the programming mode
#define BOOTSECURE2 0xA5
#define BOOTSECURE3 0x00

#define STATUS_IDLE 11			//Status
#define STATUS_BUSY 12
#define STATUS_BOOTIDLE 14
#define STATUS_BOOTBUSY 15
#define STATUS_BOOTERASE 16
#define STATUS_BOOTERROR 0x80

//SST25VF040 Serial Flash Chip command definitons
#define FLASH_READ_CMD 0x03
#define FLASH_SECT_ERASE_CMD 0x20
#define FLASH_BLOCK_ERASE_CMD 0x52
#define FLASH_CHIP_ERASE_CMD 0x60
#define FLASH_BYTE_WRITE_CMD 0x02
#define FLASH_AAI_PROG_CMD 0xAF
#define FLASH_READ_STATUS_CMD 0x05
#define FLASH_EN_WRITE_STATUS_CMD 0x50
#define FLASH_WRITE_STATUS_CMD 0x01
#define FLASH_WRITE_ENABLE_CMD 0x06
#define FLASH_WRITE_DISABLE_CMD 0x04
#define FLASH_READ_ID_CMD 0x90

//Internal EEROM address definitions
#define INTEE_CURBANK 0xFE		//1 byte signifying current program bank programmed
#define INTEE_CURMODE 0xFF		//1 byte signifying current program mode in use

#define MAX_CODESPACE 0x2000	//  4K(0x1000) for PIC16F873
#define MAX_BOOTSPACE 0x0600	//1500 words of boot codespace
// The End of the firmware loadable space
#define MAX_LOADABLE (MAX_CODESPACE-MAX_BOOTSPACE-1)
#org 0x0004, 0x07FF {}			// Consume program memory to force the loader to reside in the highest space
#org 0x0800, 0x0FFF {}
#org 0x1000, 0x17FF {}
#org 0x1800, MAX_LOADABLE {}	// BootLoader will reside from MAX_LOADABLE + 1 to the end of program memory

//FLASH address defines
//16 bit chksum is located in the top of each segment
// at PROGRAM_LENGTH-2 in the PIC code space
// all addresses are 16 bit pointers(for actual byte addreses mult by 2)
#define PROGRAM_LENGTH (MAX_CODESPACE-MAX_BOOTSPACE)
#define FLASH_PROG0_START 0x8000/2
#define FLASH_PROG1_START 0xC000/2

/*---------------------------------------------------------------------------*/
/*--------> T Y P E D E F S   U N I O N S    S T R U C T U R E S <-----------*/
/*---------------------------------------------------------------------------*/
typedef unsigned int8	U8BIT;	// Unsigned 8 bit variables
typedef signed int8		S8BIT;	// Signed 8 bit variables
typedef unsigned int16	U16BIT;	// Unsigned 16 bit variables
typedef signed int16	S16BIT;	// Signed 16 bit variables
typedef unsigned int32	U32BIT;	// Unsigned 16 bit variables
typedef signed int32	S32BIT;	// Signed 16 bit variables
typedef int1 BOOL;

union u16bits	//efficient but non-portable way to split up
{				// 16 bit variables into their individual bytes
	struct
	{
		U8BIT b0;	//access lsb as xxx.bytes.b0
		U8BIT b1;	//access msb as xxx.bytes.b1
	}bytes;
	U16BIT all;		//access as xxx.all
};

union u32bits	//efficient but non-portable way to split up
{				// 32 bit variables into their individual bytes
	struct
	{
		U8BIT b0;	//access lsb as xxx.bytes.b0
		U8BIT b1;	//access as xxx.bytes.b1
		U8BIT b2;	//access as xxx.bytes.b2
		U8BIT b3;	//access msb as xxx.bytes.b3
	}bytes;
	U32BIT all;		//access as xxx.all
};

union uMsgstruct
{			// main host/target message buffer structure
	struct
	{
		U8BIT b0;
		U8BIT b1;
		U8BIT b2;
		U8BIT b3;
		U8BIT parms[MAX_MSG-4];
	}bytes;
	struct
	{
		U16BIT w0;
		U16BIT alow;
		U16BIT ahi;
		U16BIT parms[(MAX_MSG-6)/2];
	}dwords;
	struct
	{
		U16BIT header;
		U16BIT CItem;
		U16BIT parms[(MAX_MSG-4)/2];
	}citem16;
	struct
	{
		U16BIT header;
		U16BIT CItem;
		U8BIT parms[(MAX_MSG-4)];
	}citem8;
	struct
	{
		U16BIT header;
		U16BIT CItem;
		U32BIT parms[(MAX_MSG-4)/4];
	}citem32;
	U8BIT buf[MAX_MSG];
};

struct port_a_layout
{
	int xio_0:1;
	int xio_1:1;
	int xio_2:1;
	int xio_3:1;
	int clktemp:1;
	int iosync:1;
};
struct port_a_layout port_a;
#byte port_a = 5
#define PORTA_INIT 0x1F

struct port_b_layout
{
	int swinputs:4;
	int ook:1;
	int ddsreset:1;
	int rb6:1;
	int rb7:1;
};
struct port_b_layout port_b;
#byte port_b = 6
#define PORTB_INIT 0xEF

struct port_c_layout
{
	int dds_cs:1;
	int ee_cs:1;
	int lclr:1;
	int sck:1;
	int sdi:1;
	int sdo:1;
	int tx:1;
	int rx:1;
};
struct port_c_layout port_c;
#byte port_c = 7
#define PORTC_INIT 0xD3	//1101 0011b

/*---------------------------------------------------------------------------*/
/*-------------------> M E M O R Y  A L L O C A T I O N <--------------------*/
/*---------------------------------------------------------------------------*/

#org 0x1FF9, 0x1FFF	// Place Bootcode Version at very end of code space
const U16BIT VERSION = 4; 	//lsb,msb of version/100  1.23 = 123

#reserve 0x7E,0x7F		//reserve 16 bits for inter-module boot flags
#byte Bflag0 = 0x7E
#byte Bflag1 = 0x7F

BOOL UnLocked;
BOOL GotMsg;
BOOL SendResp;
BOOL ProgOK;
BOOL StayInBoot;
BOOL CurrentProgBank1;
BOOL NeedProgBank1;
union uMsgstruct Msg;
U8BIT TmpU8;
union u16bits uTemp16;
U8BIT LEDTimer;
U8BIT CurrentMsgPos;
U8BIT MsgTimer;
U8BIT Status;
U8BIT SendMsgPos;
U8BIT SWvalue;
U8BIT CurrentMode;
U8BIT NewMode;
U16BIT ResetVectorLocation; // Downloaded reset vector is written at this address
U8BIT AddressSrcHi;	//temp variable for holding Source upper byte Address
U16BIT AddressSrc;	//temp variable for holding Source Address
U16BIT AddressDest;	//temp variable for holding Destination Address

const char ID_DESC[]= "VBBoot";

/*---------------------------------------------------------------------------*/
/*--------------------------> P R O T O T Y P E S <--------------------------*/
/*---------------------------------------------------------------------------*/
void main (void);
#separate	// Force to function to save program memory
BOOLEAN	WriteVerifyProgramEeprom (U16BIT address, U16BIT data);
#separate	// Force to function to save program memory
void WriteSPI(U8BIT data);
#separate
void WriteEnable();
#separate
void WaitBusy();
#inline
void ServiceHostMsg();
#inline
void SendResponseMsg();
#inline
BOOL CalcCheckSum();
#inline
BOOL WriteVerifyFLASH( U16BIT data);

#inline
void SectorErase(U8BIT Sector);
#inline
void BlockErase(U8BIT Block);
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                             ---------                                     */
/*                            | M a i n |                                    */
/*                             ---------                                     */
/*    This is the Main program entry point.                                  */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void main (void)
{
	INTCON = 0;		//disable/clear all interrupts since we can get here
	PIR1 = 0;		//from a jump as well as POR
	setup_adc_ports(NO_ANALOGS);
	set_tris_a(PORTA_DIR);	// set port bit directions
	set_tris_b(PORTB_DIR);
	set_tris_c(PORTC_DIR);
	output_a( PORTA_INIT );
	output_b( PORTB_INIT );
	output_c( PORTC_INIT );
	port_b_pullups(TRUE);
	setup_timer_1( T1_INTERNAL|T1_DIV_BY_1);
	setup_wdt(WDT_288MS);
	restart_wdt();
	CurrentMsgPos = 0;
	GotMsg = FALSE;
	SendResp = FALSE;
	ProgOK = FALSE;
	UnLocked = FALSE;
	Status = STATUS_BOOTIDLE;
	MsgTimer = MSG_TIMEOUT_TIME;
	LEDTimer = 0;
	SendMsgPos = 0;
	port_b.ddsreset = 0;
	port_a.iosync = 1;
	port_a.iosync = 0;
	bit_clear(RCSTA,4);
	bit_set(RCSTA,4);
	TmpU8 = RCREG;		//make sure nothing in rx buffer
	//				Switch Settings
	// SW == 0  means use the last mode stored in EEROM and ignor sw 
	// SW == 1  means use mode 1  program 0
	// SW == 2  means use mode 2  program 0
	// SW == 3  means use mode 3  program 0
	// SW == 4  means use mode 4  program 0
	// SW == 5  means use mode 5  program 0
	// SW == 6  means use mode 6  program 0
	// SW == 7  means use mode 7  program 1
	// SW == 8  means use mode 8  program 1
	// SW == 9  means use mode 9  program 1
	// SW == A-E Not defined
	// SW == F  Stay in Boot code
	// upper 4 bits are requested mode  bottom 4 bits are current mode
	if( read_eeprom(INTEE_CURBANK) != 0)
		CurrentProgBank1 = TRUE;
	else
		CurrentProgBank1 = FALSE;
	CurrentMode = read_eeprom(INTEE_CURMODE);
	CurrentMode &= 0x0F;
	SWvalue = ~input_b();
	SWvalue &= 0x0F;
	if( 0 == SWvalue )
		NewMode = CurrentMode;
	else
		NewMode = SWvalue;
	if( NewMode >= 7 )
		 NeedProgBank1 = TRUE;
	 else
		 NeedProgBank1 = FALSE;

	//	setup_spi( SPI_MASTER|SPI_H_TO_L|SPI_CLK_DIV_4 );
	//  library setup doesnt implement Lo to Hi clock correctly
	SSPSTAT  = 0x40;	//0100 0000b
	SSPCON  = 0x20;		//0010 0000b

	//if checksum fails or switch == 15	then enter boot code
	StayInBoot = FALSE;
	if( !CalcCheckSum() || SWvalue==0x0F )
	{
		StayInBoot = TRUE;
	}

	//if entered from FLASH code with boot flag set the enter boot code
	if( (Bflag0==BOOTFLAG0) && (Bflag1==BOOTFLAG1) )
	{		//need to ack enter boot message?
		Bflag0=0; Bflag1=0;
		Msg.bytes.b0 = CI_UPDATE_MODE_CONTROL_SETRESPLEN;
		Msg.bytes.b1 = TYPE_TARG_RESP_CITEM;
		Msg.citem8.CItem = CI_UPDATE_MODE_CONTROL;
		Msg.citem8.parms[0] = 0;
		Msg.citem8.parms[1] = CI_UPDATE_MODE_CONTROL_ENTER;
		Msg.citem8.parms[2] = BOOTSECURE0;
		Msg.citem8.parms[3] = BOOTSECURE1;
		Msg.citem8.parms[4] = BOOTSECURE2;
		Msg.citem8.parms[5] = BOOTSECURE3;
		SendResp = TRUE;
		StayInBoot = TRUE;
	}
	//if requested program bank matches current program bank
	// then jump to PIC FLASH application code
	if( (NeedProgBank1 == CurrentProgBank1) && !StayInBoot)
	{
		if(CurrentMode != NewMode)
			write_eeprom(INTEE_CURMODE, NewMode);
		// Jump to the Application code by processing it's reset vector
		NewResetVect:
		#asm
		MOVLW	0x00	// Self-modifying code
		MOVWF	0x0A	//The PIC FLASH reset entry vector will be written here
		GOTO	0x00	//whenever the Flash is updated
		NOP
		NOP
		#endasm
	}
	
	while(StayInBoot)	//if need to stay in boot code to await new host downloads
	{
		if(RCSTA&0x07)	//if any UART errors
		{
			bit_clear(RCSTA,4);
			bit_set(RCSTA,4);
			TmpU8 = RCREG;
		}
		while( kbhit() )	//read any pending serial msg bytes
		{
			Msg.buf[CurrentMsgPos++] = RCREG;	//read a byte
			if(CurrentMsgPos > MAX_MSG)
				CurrentMsgPos--;	//clamp msg length to buffer size
			if(CurrentMsgPos >= Msg.bytes.b0)
			{
				CurrentMsgPos = 0;	//got a complete msg
				GotMsg = TRUE;
			}
		}
		if(GotMsg) 		// if a msg received from Host
			ServiceHostMsg();
		if(SendResp)	//if need to respond back to Host
			SendResponseMsg();
		if( TMR1IF)	//process message error timer at Timer1 65mSec rate
		{
			TMR1IF = 0;
			if(CurrentMsgPos != 0)	//if a msg is being received
			{
				if(--MsgTimer == 0 )	//if msg not fully received
				{						// then reset msg pointer
					CurrentMsgPos = 0;
					MsgTimer = MSG_TIMEOUT_TIME;
					Status = STATUS_BOOTIDLE;
					UnLocked = FALSE;
				}
			}
			else
			{
				MsgTimer = MSG_TIMEOUT_TIME;
			}
			if(UnLocked)
				port_c.lclr = bit_test(LEDTimer++,1);
			else
				port_c.lclr = bit_test(LEDTimer++,4);
		}
		restart_wdt();
	}
	//  Here to program PIC with specified code from external FLASH
	// Self-modifying code support - Get the address where the downloaded
	//		 reset vector will be written
	ProgOK = TRUE;
	ResetVectorLocation = label_address(NewResetVect);
	if( NeedProgBank1 )
		AddressSrc = FLASH_PROG1_START;	//if need to load Program 1
	else
		AddressSrc = FLASH_PROG0_START;	//if need to load Program 0

	for( AddressDest=0; AddressDest<PROGRAM_LENGTH; AddressDest++)
	{
		// Get the next program data word at Address
		uTemp16.all = AddressSrc<<1;	//make byte address
		port_c.ee_cs = 0;		//EEPROM chip select active LOW
		WriteSPI(FLASH_READ_CMD);
		WriteSPI( 0 );
		WriteSPI( uTemp16.bytes.b1 );
		WriteSPI( uTemp16.bytes.b0 );
		uTemp16.bytes.b0 = spi_read(0);
		uTemp16.bytes.b1 = spi_read(0);
		port_c.ee_cs = 1;
		if( AddressDest <= 0x0003 )
		{	//intercept reset vector and write it in the boot area instead
			if(!WriteVerifyProgramEeprom(AddressDest+ResetVectorLocation, uTemp16.all) )
			{
				ProgOK = FALSE;
				break;
			}
		}
		else
		{	//just copy from external EEROM to the PIC FLASH program area
			if(!WriteVerifyProgramEeprom(AddressDest, uTemp16.all) )
			{
				ProgOK = FALSE;
				break;
			}
		}
		AddressSrc++;
		if( TMR1IF)	//toggle LED at 65mSec rate while programming
		{
			TMR1IF = 0;
			port_c.lclr = ~port_c.lclr;
		}
		restart_wdt();
	}
	//if all went ok then write current program Bank ID to internal PIC eedata area
	if( ProgOK )
	{
		if( NeedProgBank1 )
			write_eeprom(INTEE_CURBANK, 1);
		else
			write_eeprom(INTEE_CURBANK, 0);
	}
	port_c.lclr = 0;
	reset_cpu ();	//reset PIC to re-enter and hopefully pass checksum and execute
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                    -----------------------------                          */
/*                   | S e r v i c e H o s t M s g |                         */
/*                    -----------------------------                          */
/*    This routine is called to parse a new message from the Host.           */
/* INPUTS: The Msg Buffer containing the new msg from the Host               */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
#inline
void ServiceHostMsg()
{
U8BIT i;
	if( Msg.bytes.b1 == TYPE_HOST_SET_CITEM )
	{	// Is a message from host to set a control item value
		Msg.bytes.b1 = TYPE_TARG_RESP_CITEM;
		switch(Msg.citem8.CItem)
		{
			case CI_UPDATE_MODE_CONTROL:
				if( (Msg.bytes.parms[2] == BOOTSECURE0 ) &&
						(Msg.bytes.parms[3] == BOOTSECURE1 ) &&
						(Msg.bytes.parms[4] == BOOTSECURE2 ) &&
						(Msg.bytes.parms[5] == BOOTSECURE3 ) )
				{	//is valid boot command
					TmpU8 = Msg.bytes.parms[0];	//get destination/sector parameter
					if( Msg.bytes.parms[1] == CI_UPDATE_MODE_CONTROL_START )
					{
						Status = STATUS_BOOTBUSY;
						UnLocked = TRUE;	//allow programming blocks
						ProgOK = TRUE;
					}
					else if( Msg.bytes.parms[1] == CI_UPDATE_MODE_CONTROL_ERASE)
					{
						Status = STATUS_BOOTERASE;
						port_c.ee_cs = 0;
						WriteSPI(FLASH_EN_WRITE_STATUS_CMD);
						port_c.ee_cs = 1;
						port_c.ee_cs = 0;
						WriteSPI(FLASH_WRITE_STATUS_CMD);	//make sure external FLASH is writable
						WriteSPI(0);
						port_c.ee_cs = 1;
						if(TmpU8>=0xF0)
							BlockErase(TmpU8);
						else
							SectorErase(TmpU8);
					}
					else if( (Msg.bytes.parms[1] == CI_UPDATE_MODE_CONTROL_END) && UnLocked && ProgOK)
					{	//is end of programming sequence
						if( (TmpU8<2) || (TmpU8==4) )	//if was data table load
						{
							while(TRUE){};	//just let wdt reset PIC to re-enter
						}
						else
						{
							if( TmpU8 == 2 )			//bank 0
								NeedProgBank1 = FALSE;
							else
								NeedProgBank1 = TRUE;
							StayInBoot = FALSE;	//need to continue and program PIC
							UnLocked = FALSE;
						}
					}
					else
					{
						UnLocked = FALSE;
						Status = STATUS_BOOTERROR;
					}
				}
				else
				{
					Msg.citem8.header = TARG_RESP_NAK;	// Send NAK 
				}
				break;
			default:
				Msg.citem8.header = TARG_RESP_NAK;	// Send NAK 
				break;
		}
	}
	else if( Msg.bytes.b1 == TYPE_HOST_REQ_CITEM )
	{	// Is a message from host requesting a control item value
		Msg.bytes.b1 = TYPE_TARG_RESP_CITEM;
		switch(Msg.citem8.CItem )
		{						 
			case CI_GENERAL_INTERFACE_NAME:
				i = 0;
				do{
					Msg.bytes.parms[i] = ID_DESC[i];
				}while(Msg.bytes.parms[i++] != 0);
				Msg.bytes.b0 = 4+i;
				break;
			case CI_GENERAL_INTERFACE_VERSION:
				Msg.bytes.b0 = CI_GENERAL_INTERFACE_VERSION_SETRESPLEN;
				Msg.bytes.parms[0] = (ASCP_INTERFACE_VERSION&0x00FF);
				Msg.bytes.parms[1] = ((ASCP_INTERFACE_VERSION>>8)&0x00FF);
				break;
			case CI_GENERAL_HARDFIRM_VERSION:
				Msg.bytes.b0 = CI_GENERAL_HARDFIRM_VERSION_SETRESPLEN;
				if(Msg.bytes.parms[0] == 0)
				{
					Msg.bytes.parms[1] = make8(VERSION,0);
					Msg.bytes.parms[2] = make8(VERSION,1);
				}
				else
				{
					Msg.bytes.parms[1] =  0;
					Msg.bytes.parms[2] =  0;
				}
				break;
			case CI_GENERAL_STATUS_CODE:
				Msg.bytes.b0 = CI_GENERAL_STATUS_CODE_SETRESPLEN;
				Msg.bytes.parms[0] = Status;
				break;
			default:
				Msg.citem8.header = TARG_RESP_NAK;	// Send NAK 
				break;
		}
	}
	else if( Msg.bytes.b1 == TYPE_HOST_DATA_ITEM0 )
	{	//got a block of data to program into the External EEROM
		if( UnLocked )
		{
			TmpU8 = (Msg.bytes.b0-6)/2;	 // number of 16 bit words to program
			AddressSrcHi = make8(Msg.dwords.ahi,0); //A23 to A16 starting address to program 
			AddressSrcHi <<= 1;	//make byte address
			AddressSrc = Msg.dwords.alow;	 //A15 to A0 starting address to program 
			if( bit_test(AddressSrc,15) )
				AddressSrcHi++;
			AddressSrc <<= 1;
			for( i=0; i<TmpU8; i++)
			{
				if(!WriteVerifyFLASH( Msg.dwords.parms[i]) )
				{
					ProgOK = FALSE;
					break;
				}
				AddressSrc+=2;
				if(AddressSrc == 0)
					AddressSrcHi++;
			}
		}
		if(ProgOK)
		{
			Msg.bytes.b0 = DATA_ITEM_ACK_LENGTH;
			Msg.bytes.b1 = TYPE_DATA_ITEM_ACK;
			Msg.bytes.b2 = 0;	//ACK DAta Item0
		}
		else
		{
			Status = STATUS_BOOTERROR;
			Msg.bytes.b0 = CI_GENERAL_STATUS_CODE_SETRESPLEN;
			Msg.bytes.b1 = TYPE_TARG_UNSOLICITED_CITEM; 
			Msg.citem8.CItem = CI_GENERAL_STATUS_CODE;
			Msg.bytes.parms[0] = Status;
		}
	}
	SendResp = TRUE;
	GotMsg = FALSE;
	SendMsgPos = 0;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                     -------------------------------                       */
/*                    | S e n d R e s p o n s e M s g |                      */
/*                     -------------------------------                       */
/*    This routine is called to send a response to the Host.                 */
/* INPUTS: The Msg Buffer containing the msg to send back to the Host.       */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
#inline
void SendResponseMsg()
{
	while( SendMsgPos<Msg.bytes.b0 )	//Send resp buffer
		putc( Msg.buf[SendMsgPos++] );
	SendMsgPos = 0;
	SendResp = FALSE;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                     -------------------------                             */
/*                    | C a l c C h e c k S u m |                            */
/*                     -------------------------                             */
/*    This routine calculates the program ROM checksum and tests it against  */
/* The last word in firmware space.  Returns TRUE if OK                      */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
#inline
BOOL CalcCheckSum()
{
U16BIT sum;
U16BIT adr;
	sum = 0;	//sum
	for(adr=4; adr<PROGRAM_LENGTH-1; adr++)
		sum += read_program_eeprom(adr);
	sum &= 0x3FFF;
	return( sum==read_program_eeprom(adr) );
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                    -----------------------                                */
/*                   | S e c t o r E r a s e |                               */
/*                    -----------------------                                */
/*    This routine is called to Erase a 16Kbyte sector of the external FLASH */
/* Sector address is a single byte specifying the start of the sector.       */
/*  0x00 = erase 0x000000 to 0x003FFF                                        */
/*  0x40 = erase 0x004000 to 0x007FFF                                        */
/*  0x80 = erase 0x008000 to 0x00CFFF                                        */
/*  0xC0 = erase 0x00C000 to 0x00FFFF                                        */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
#inline
void SectorErase(U8BIT Sector)
{
	for( TmpU8=0; TmpU8<4; TmpU8++)
	{
		WriteEnable();
		port_c.ee_cs = 0;
		WriteSPI(FLASH_SECT_ERASE_CMD);
		WriteSPI(0);
		WriteSPI(Sector);
		WriteSPI(0);
		port_c.ee_cs = 1;
		WaitBusy();
		Sector += 0x10;
	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                    ---------------------                                  */
/*                   | B l o c k E r a s e |                                 */
/*                    ---------------------                                  */
/*    This routine is called to Erase the external serial Flash              */
/*        From 0x010000 to 0x07FFFF using the block erase command            */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
#inline
void BlockErase(U8BIT Block)
{
U8BIT numblocks;
	uTemp16.bytes.b1 = Block & 0x0F;
	uTemp16.bytes.b0 = 0;
	if(	uTemp16.bytes.b1 == 1 )
		numblocks = 8;		   //start at 0x10000 and erase 8-32K blocks
	else
		numblocks = 6;		   //start at 0x50000 and erase 6-32K blocks
	for( TmpU8=0; TmpU8<numblocks; TmpU8++)
	{
		WriteEnable();
		port_c.ee_cs = 0;
		WriteSPI(FLASH_BLOCK_ERASE_CMD);
		WriteSPI(uTemp16.bytes.b1);
		WriteSPI(uTemp16.bytes.b0);
		WriteSPI(0);
		port_c.ee_cs = 1;
		WaitBusy();
		uTemp16.all += 0x0080;
	} 
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*             -------------------------------------------------             */
/*            | W r i t e V e r i f y P r o g r a m E e p r o m |            */
/*             -------------------------------------------------             */
/*    This routine is called to Program a single word into program memory.   */
/* If the current data is the same, the programming is skipped.              */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
#separate
BOOL WriteVerifyProgramEeprom(U16BIT address, U16BIT data)
{
U16BIT currentdata;
	if(address>=0x2000) //make sure doesnt get out of code space
		return FALSE;
	bit_clear(data,15);
	bit_clear(data,14);
	// Get the data that is currently at the target address
	currentdata = read_program_eeprom (address); 
	if(currentdata != data)
	{	// Write to the PIC program FLASH only if the data is not the same
		write_program_eeprom (address, data);	//write to Program FLASH
		currentdata = read_program_eeprom (address);
  		if (currentdata != data)	// verify it
			return (FALSE);
	}
	return (TRUE);
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                     ---------------------------------                     */
/*                    | W r i t e V e r i f y F L A S H |                    */
/*                     ---------------------------------                     */
/*    This routine is called to Program a single word into external FLASH.   */
/* If the current data is the same, the programming is skipped.              */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
#inline
BOOL WriteVerifyFLASH( U16BIT data)
{
union u16bits uCurrentData;
	// Get the data that is currently at address
	port_c.ee_cs = 0;		//EEPROM chip select active LOW
	WriteSPI(FLASH_READ_CMD);
	WriteSPI( AddressSrcHi );
	WriteSPI( make8(AddressSrc,1) );
	WriteSPI( make8(AddressSrc,0) );
	uCurrentData.bytes.b0 = spi_read(0);
	uCurrentData.bytes.b1 = spi_read(0);
	port_c.ee_cs = 1;
	if(uCurrentData.all != data)
	{	// Write to the external eeprom only if the data is not the same
		WriteEnable();
		port_c.ee_cs = 0;
		WriteSPI(FLASH_BYTE_WRITE_CMD);
		WriteSPI( AddressSrcHi );
		WriteSPI( make8(AddressSrc,1) );
		WriteSPI( make8(AddressSrc,0) );
		WriteSPI( make8(data,0) );
		port_c.ee_cs = 1;
		WaitBusy();
		WriteEnable();
		port_c.ee_cs = 0;
		WriteSPI(FLASH_BYTE_WRITE_CMD);
		WriteSPI( AddressSrcHi );
		WriteSPI( make8(AddressSrc,1) );
		WriteSPI( make8(AddressSrc,0) + 1 );
		WriteSPI( make8(data,1) );
		port_c.ee_cs = 1;
		WaitBusy();
		port_c.ee_cs = 0;
		WriteSPI(FLASH_READ_CMD);
		WriteSPI( AddressSrcHi );
		WriteSPI( make8(AddressSrc,1) );
		WriteSPI( make8(AddressSrc,0) );
		uCurrentData.bytes.b0 = spi_read(0);
		uCurrentData.bytes.b1 = spi_read(0);
		port_c.ee_cs = 1;
  		if(uCurrentData.all != data)	// verify it
			return (FALSE);
	}
	return (TRUE);
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                       -----------------                                   */
/*                      | W a i t B u s y |                                  */
/*                       -----------------                                   */
/*    This routine waits for the BUSY bit to clear.                          */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
#separate
void WaitBusy()
{
U8BIT bsy;
 	port_c.ee_cs = 0;
 	do
 	{
	 	spi_write(FLASH_READ_STATUS_CMD);
 		bsy = spi_read(0);
 	}while( bit_test(bsy,0) );
 	port_c.ee_cs = 1;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                       -----------------------                             */
/*                      | W r i t e E n a b l e |                            */
/*                       -----------------------                             */
/*    This routine enables Flash Memory writes.                              */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
#separate
void WriteEnable()
{
	port_c.ee_cs = 0;
 	spi_write(FLASH_WRITE_ENABLE_CMD);
	port_c.ee_cs = 1;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                       -----------------                                   */
/*                      | W r i t e S P I |                                  */
/*                       -----------------                                   */
/*    This routine encapsulates the library function to save space.          */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
#separate
void WriteSPI(U8BIT data)
{
 	spi_write(data);
}
// End of Bootloader file
