/* ==========================================================================*/
/* - - - - - - - - - -   V e r s a B _ F S K . C   - - - - - - - - - - - - - */
/* ==========================================================================*/
/*	Created   1-18-2004   M. Wheatley                                        */
/*	Modified Mar 23, 2004  MW                                                */
/*.........................................................................  */
/* FSK441,SSTV, FM voice implementation for the VersaBeacon Project.                   */
/*---------------------------------------------------------------------------*/
/*------------------------> I N C L U D E S <--------------------------------*/
/*---------------------------------------------------------------------------*/
#case		// Make the compiler case sensitive
#include <16F876.h>
#NOLIST
#include <stdlib.h>
#include <tables.h>
#LIST

#define SHORT_DISPLAY 1
/*---------------------------------------------------------------------------*/
/*----------------> P R O C E S S O R    S P E C I F I C <-------------------*/
/*---------------------------------------------------------------------------*/
#define CLOCK_FREQ 6000000	// PIC Oscillator Frequency

//NOTE!!! New version compiler 3.182 decided to define 
// Program EEROM Write Enable as NOWRT!!
//   Beware it may change back someday.
#fuses HS,PUT,NOBROWNOUT,WDT,NOPROTECT,NOLVP,NOWRT,NOCPD,NODEBUG
#use delay(clock=CLOCK_FREQ)
#use rs232 (baud=19200, xmit=PIN_C6, rcv=PIN_C7)  // Use the hardware USART
#use fast_io(A)
#use fast_io(B)
#use fast_io(C)

/*    Define the input/output port pins for the 877 based interface          */
#define PORTA_DIR 0x1F	/* port A direction TRIS bits   (1==INPUT)           */
#define PORTB_DIR 0xCF	/* port B direction TRIS bits                        */
#define PORTC_DIR 0x90	/* port C direction TRIS bits                        */
#define PORTC_I2CDIR 0x98	/* port C direction TRIS bits for i2c            */

#byte SSPBUF  = 0x13	//define the serial port registers
#byte SSPCON  = 0x14	// since the built in functions aren't used
#byte SSPCON2 = 0x91
#byte SSPADD = 0x93
#byte SSPSTAT  = 0x94
#byte TXSTA  = 0x98
#byte TXREG  = 0x19
#byte RCSTA  = 0x18
#byte RCREG  = 0x1A
#byte SPBRG  = 0x99

#byte SSPSTAT  = 0x94
#byte SSPBUF  = 0x13
#byte SSPCON  = 0x14
#byte SSPCON2 = 0x91
 #define SEN 0
 #define PEN 2
 #define RCEN 3
 #define ACKEN 4
 #define ACKDT 5
 #define ACKSTAT 6
#byte SSPADD = 0x93

#byte PIR1  = 0x0C
#define SSPIF 3

#byte PR2 = 0x92

#byte ADCON1 = 0x9F
#BIT TMR2IF = 0x0C.1


/*---------------------------------------------------------------------------*/
/*--------> T Y P E D E F S   U N I O N S    S T R U C T U R E S <-----------*/
/*---------------------------------------------------------------------------*/
typedef unsigned int8	U8BIT;	// Unsigned 8 bit variables
typedef signed int8		S8BIT;	// Signed 8 bit variables
typedef unsigned int16	U16BIT;	// Unsigned 16 bit variables
typedef signed int16	S16BIT;	// Signed 16 bit variables
typedef unsigned int32	U32BIT;	// Unsigned 16 bit variables
typedef signed int32	S32BIT;	// Signed 16 bit variables
typedef int1 BOOL;

union u16bits	//efficient but non-portable way to split up
{				// 16 bit variables into their individual bytes
	struct
	{
		U8BIT b0;	//access lsb as xxx.bytes.b0
		U8BIT b1;	//access as xxx.bytes.b1
	}bytes;
	U16BIT all;		//access as xxx.all
};

union u32bits	//efficient but non-portable way to split up
{				// 32 bit variables into their individual bytes
	struct
	{
		U8BIT b0;	//access lsb as xxx.bytes.b0
		U8BIT b1;	//access as xxx.bytes.b1
		U8BIT b2;	//access as xxx.bytes.b2
		U8BIT b3;	//access msb as xxx.bytes.b3
	}bytes;
	U32BIT all;		//access as xxx.all
};

union u40bits	//efficient but non-portable way to split up
{				// 40 bit variables into their individual bytes
	struct
	{
		U8BIT b0;	//access lsb as xxx.bytes.b0
		U8BIT b1;	//access as xxx.bytes.b1
		U8BIT b2;	//access as xxx.bytes.b2
		U8BIT b3;	//access as xxx.bytes.b3
		U8BIT b4;	//access msb as xxx.bytes.b4
	}bytes;
	U32BIT all;		//access as xxx.all
};
//  I/O Port definitions and data structures
struct port_a_layout
{
	int xio_0:1;		//external io bit 0
	int xio_1:1;		//external io bit 1
	int xio_2:1;		//external io bit 2
	int xio_3:1;		//external io bit 3
	int clktemp:1;		//onboard temperature sensor analog input
	int iosync:1;		//AD6654 IOSYNC and I2C/SPI multiplex select
	int nc6:1;
	int nc7:1;
};
struct port_a_layout port_a;
#byte port_a = 5
#define PORTA_INIT 0x1F

struct port_b_layout
{
	int swinputs:4;		//rotary dipsw Binary inputs 0 to 15
	int ddsreset:1;		//DDS reset active high
	int ook:1;			//On/Off keying output to AD6654
	int rb6:1;			//undefined
	int rb7:1;			//hold low during reset to enter boot mode
};
struct port_b_layout port_b;
#byte port_b = 6
#define PORTB_INIT 0xCF

struct port_c_layout
{
	int dds_cs:1;		//DDS chip select active LOW
	int ee_cs:1;		//External EEROM chip select active LOW
	int lclr:1;			//latch clear / LED control
	int sck:1;			//SPI clock / I2C SCL
	int sdi:1;			//SPI data in / I2C SDA
	int sdo:1;			//API Data Out
	int tx:1;			// RS232 TX
	int rx:1;			// RS232 RX
};
struct port_c_layout port_c;
#byte port_c = 7
#define PORTC_INIT 0xD7	//1101 0111b

/*---------------------------------------------------------------------------*/
/*--------------------> L O C A L   D E F I N E S <--------------------------*/
/*---------------------------------------------------------------------------*/
#define TIMER_TICS_PER_SECOND (CLOCK_FREQ/(4*256*256))

#define BOOTFLAG0 0x5A	//when reset, if location 0x7E,0x7F equal 0x5AA5
#define BOOTFLAG1 0xA5	//then it means that it should jump into the programming
						// mode.
#define MAX_STR 32		//max characters in parsing string
#define PARSE_TOP 0		//ASCII input parsing levels
#define PARSE_TIME 1
#define PARSE_GETNUMBER 2
#define PARSE_GETSTRING 3
#define MAX_VARSTR 6  	//max characters in variable data string

#define PARM_FREQ_0 0		//defines for data entry destinations
#define PARM_REFFREQ 1
#define PARM_MIN 2
#define PARM_HR 3
#define PARM_DAY 4
#define PARM_DATE 5
#define PARM_MON 6
#define PARM_YR 7
#define PARM_CLKCAL 8
#define PARM_WPM 9
#define PARM_MODE 10
#define PARM_DUTY 11

//SST25VF040 Serial Flash Chip command definitons
#define FLASH_READ_CMD 0x03
#define FLASH_SECT_ERASE_CMD 0x20
#define FLASH_BLOCK_ERASE_CMD 0x52
#define FLASH_CHIP_ERASE_CMD 0x60
#define FLASH_BYTE_WRITE_CMD 0x02
#define FLASH_AAI_PROG_CMD 0xAF
#define FLASH_READ_STATUS_CMD 0x05
#define FLASH_EN_WRITE_STATUS_CMD 0x50
#define FLASH_WRITE_STATUS_CMD 0x01
#define FLASH_WRITE_ENABLE_CMD 0x06
#define FLASH_WRITE_DISABLE_CMD 0x04
#define FLASH_READ_ID_CMD 0x90

#define DDS_CFR1 0
 #define DDS_OFF 0x0C
#define DDS_CFR2 1
#define DDS_ASF 2
#define DDS_ARR 3
#define DDS_FTW0 4
#define DDS_POW0 5
#define DDS_FTW1 6
#define DDS_RSCW0 7
#define DDS_NLSCW 7
#define DDS_RSCW1 8
#define DDS_PLSCW 8
#define DDS_RSCW2 9
#define DDS_RSCW3 10
#define DDS_RAM 11

#define I2CREAD_TOD 0xD1
#define I2CWRITE_TOD 0xD0
#define TOD_SEC 1
#define TOD_MIN 2
#define TOD_HR 3
#define TOD_DAY 4
#define TOD_DATE 5
#define TOD_MON 6
#define TOD_YR 7
#define TOD_CTRL 8
 #define FT_BIT (1<<6)
#define TOD_WATCHDOG 9
#define TOD_ALM_MON 10
#define TOD_ALM_DATE 11
#define TOD_ALM_HR 12
 #define HALT_BIT (1<<6)
#define TOD_ALM_MIN 13
#define TOD_ALM_SEC 14
#define TOD_FLAGS 15
#define TOD_SQW 19

//Internal EEROM address definitions
#define INTEE_FREQ_BASE 0	//4 bytes per frequency per mode( (mode-1)*4==index )
#define INTEE_REFFREQ 88	//4 bytes of reference frequency
#define INTEE_MODE 92		//1 byte mode
#define INTEE_CWWPM 94		//1 byte CW WPM rate
#define INTEE_CWDUTY 95		//1 byte CW duty cycle factor
#define INTEE_MSG0 96		//up to 80 characters of text message
#define INTEE_MSG1 176		//up to 80 characters of text message
#define INTEE_CURBANK 0xFE	//1 byte signifying current program bank programmed
#define INTEE_CURMODE 0xFF	//1 byte signifying current program mode in use

//External FLASH address definitions
#define FLASH_TABLE0 0x0000
#define FLASH_TABLE1 0x4000
 #define FLASH_TABLE1_PSK31SHAPE 0x4000	//128 bytes(64 words of cosine table)
 #define FLASH_TABLE1_HELP0 0x4C00			//help0 string 512 bytes below help1
 #define FLASH_TABLE1_HELP1 0x4E00			//help1 string top 512 bytes of table1
#define FLASH_SSTVDATA_TABLE 0x10000
#define FLASH_VOICEDATA_TABLE 0x50000

//Mode definitions
#define MODE_ON 1		//program 0 modes
#define MODE_OOKCW 2
#define MODE_FSKCW 3
#define MODE_FMCW 4
#define MODE_PSK31 5
#define MODE_RTTY 6
//
#define MODE_FSK441 7	//program 1 modes
#define MODE_SSTV 8
#define MODE_VOICE 9
//
#define MODE_UNDEF10 10
#define MODE_UNDEF11 11
#define MODE_UNDEF12 12
#define MODE_UNDEF13 13
#define MODE_UNDEF14 14
//

#define	SYM_PER_CHAR 3
#define SYM0_FREQ 882
#define SYM1_FREQ 1323
#define SYM2_FREQ 1764
#define SYM3_FREQ 2205

#define STATE_STOPPED 0
#define STATE_START1 1
#define STATE_START2 2
#define STATE_FIRSTSYNC 3
#define STATE_VIS 4
#define STATE_SYNC 5
#define STATE_RED 6
#define STATE_GREEN 7
#define STATE_BLUE 8

#define SYNC_FREQ 1200
#define VIS_0_FREQ 1300
#define VIS_1_FREQ 1100
#define BLACK_FREQ 1500
#define MID_FREQ 1900
#define WHITE_FREQ 2300

#define PIXPERLINE 320
#define NUMLINES 256-1
#define SYMBPERSYNC 18-1
#define SYMBPERVIS 60

#define PR2_PIXRELOAD 217-1		//2283.105 Hz
#define PR2_SYNCRELOAD 250-1	//2000 Hz

#define VIS_S1 0x3C
/*---------------------------------------------------------------------------*/
/*-------------------> M E M O R Y  A L L O C A T I O N <--------------------*/
/*---------------------------------------------------------------------------*/
#reserve 0x7E,0x7F		//reserve 16 bits for inter-module boot flag
#byte Bflag0 = 0x7E
#byte Bflag1 = 0x7F

//general temp variables
U8BIT TmpU8;
S16BIT TmpS16;
union u32bits uTmpU32;
union u16bits uTmpU16;
union u16bits TXcrc;
union u40bits DDSReg;	 //32 bit register value
U32BIT BlackBaseFreq;

BOOL  GeneralTmrIRQ;	//1 second timer
U8BIT GenTimerCount;

BOOL KeyboardEntryModeActive;
U8BIT InChar;
U8BIT ParseLevel;		//parsing variables
U8BIT ParseIndex;		//current input index into parsing string
U8BIT sBuf[MAX_STR];	//string parsing buffer
U8BIT ParamPtr;			//indicates which variable is to receive the user input
U8BIT VarStr[MAX_VARSTR]; 	//string buffer for variable text data

union u32bits Frequency;

BOOL XmitOn;
U8BIT QHead;
U8BIT QTail;
U8BIT Cptr;
U8BIT VarDataPtr;
U8BIT Mode;
U8BIT EchoChar;
U8BIT LEDtimer;

U8BIT SymCount;
U8BIT SymAcc;
U8BIT SSTVState;
U32BIT SymbFreqTbl[6];
U8BIT TXShiftReg;
U8BIT BitCount;
U8BIT LineCount;
U8BIT ClkAdjCounter;
U8BIT ClkAdjVal;
U16BIT PixCount;
union u32bits ImagePointer;

union u32bits RefFreq_2; //value of DDS clock frequency divided by 2

/*---------------------------------------------------------------------------*/
/*--------------------------> P R O T O T Y P E S <--------------------------*/
/*---------------------------------------------------------------------------*/
void main (void);
#inline void ParseLevel0(char ch);
#inline void ParseTime(char ch);
#inline void GetNumber(char ch);
#inline void GetString(char ch);
#inline void PutParameter();
#inline void CheckSpecialChars();
void Prompt();
void CRLF();
void DisplayTime();
void OutputHEX(char delim);
void CalcFTW(S32BIT offset);
void Div32by32();
void InitSPI();
void InitI2C();
void WaitACK();
#inline U8BIT GetNextChar();
void InitDDS();
void WriteDDS(U8BIT adr, U8BIT numbytes);
void ReadDDS(U8BIT adr);
void ReadFLASHbase(U16BIT adr);
U8BIT ReadIntEE(U8BIT adr);
void WriteIntEE(U8BIT adr, U8BIT data);
void WriteTOD(U8BIT adr, U8BIT data);
void ReadTOD(U8BIT adr);
void ReadTemperature();
void LoadDDSTable(U16BIT EEadr, S32BIT offset);
void ServiceFSK441();
void StartSSTV();
void ServiceSSTV();
void SetPixelFreq();

/*---------------------------------------------------------------------------*/
/*----------> I N T E R R U P T    S E R V I C E    R O U T I N E S <--------*/
/*---------------------------------------------------------------------------*/

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                     -------------------                                   */
/*                    | T i m e r 1 I S R |                                  */
/*                     -------------------                                   */
/*  Timer1 Interrupt Service routine. rate is OSC/(4*256*256)-->T=0.0655 Sec */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
#INT_TIMER1
void Timer1ISR()	// the 16 bit timer1 overflows
{
	if(GenTimerCount++ == 90 )
	{
		GenTimerCount = 0;
		GeneralTmrIRQ = TRUE;
	}
}

////////////////////// Functions //////////////////////
void main (void)
{
	restart_wdt();
	setup_adc(ADC_CLOCK_DIV_32);
	setup_adc_ports(RA0_RA1_RA3_ANALOG);
	set_adc_channel(3);
//
	set_tris_a(PORTA_DIR);	// set port bit directions
	set_tris_b(PORTB_DIR);
	set_tris_c(PORTC_DIR);
	port_b_pullups(TRUE);
	output_a( PORTA_INIT );
	output_b( PORTB_INIT );
	output_c( PORTC_INIT );
	//
	GenTimerCount = 0;
	GeneralTmrIRQ = FALSE;
	XmitOn = TRUE;
	KeyboardEntryModeActive = FALSE;
	SymCount = 15;
	SymAcc = ' ';
	SSTVState = STATE_STOPPED;
	ClkAdjCounter = 0;
	ClkAdjVal = 3;
	//
	ParseLevel = PARSE_TOP;
	ParamPtr = 0;
	//
	Mode = ReadIntEE(INTEE_CURMODE);
	if( Mode<MODE_FSK441 )
	{						// in wrong program bank for some reason
		while(TRUE){};		//let watchdog timer reset CPU
	}
	if( Mode>MODE_VOICE )
		Mode = MODE_VOICE;
	InitSPI();
	setup_timer_1( T1_INTERNAL|T1_DIV_BY_1);
	enable_interrupts(INT_TIMER1);
	InitDDS();
	TMR2IF = FALSE;
	bit_clear(RCSTA,4);
	bit_set(RCSTA,4);
	TmpU8 = RCREG;
	TmpU8 = ReadTOD( TOD_ALM_HR);
	TmpU8 &= HALT_BIT;
	WriteTOD( TOD_ALM_HR, TmpU8);
	Prompt();
	while(TRUE)
	{
		if(RCSTA&0x07)	//if any serial port errors
		{
			bit_clear(RCSTA,4);
			bit_set(RCSTA,4);
			TmpU8 = RCREG;
		}
		while( kbhit() )	//read any pending serial msg bytes
		{
			InChar = RCREG;	//read a byte
			CheckSpecialChars();	//process any special characters
			if(!KeyboardEntryModeActive)	//if not keyboard entry mode
			{
				putc(InChar);	//echo character
			}
			else	//if in keyboard mode stick into character buffer
			{
				if(InChar == '\b')	//Backspace
				{
					if(QHead != QTail)
						QHead--;
					else
						sBuf[QHead++] = InChar;
				}
				else
				{
					sBuf[QHead++] = InChar;
					if(QHead == QTail)
						QHead--;
				}
				QHead &= 0x1F;
			}
			switch(	ParseLevel )	//enter into parsing stages
			{
				case PARSE_TOP:
					ParseLevel0(InChar);
					break;
				case PARSE_TIME:
					ParseTime(InChar);
					break;
				case PARSE_GETNUMBER:
					GetNumber(InChar);
					break;
				case PARSE_GETSTRING:
					GetString(InChar);
					break;
				default:
					break;
			}
			if(!KeyboardEntryModeActive)
			{
				if(InChar=='\r')
					Prompt();		//send prompt if return key hit
			}
		}
		// here to process stuff
		if(	XmitOn && (ParseLevel==PARSE_TOP) )
		{
			if( Mode==MODE_VOICE )
			{	//this loops at about a 6550Hz sample rate
//				DDSReg.all = BlackBaseFreq + (U32BIT)read_adc();
				port_c.lclr = 1;
				port_c.ee_cs = 0;
				spi_write(FLASH_READ_CMD);
				spi_write( uTmpU32.bytes.b2 );
				spi_write( uTmpU32.bytes.b1 );
				spi_write( uTmpU32.bytes.b0 );
				uTmpU16.bytes.b1 = spi_read(0);
				port_c.ee_cs = 1;
				port_c.dds_cs = 0;
				SSPBUF = DDS_FTW0;	//direct write to pipeline next code
				DDSReg.all = BlackBaseFreq + (U32BIT)(uTmpU16.all);
				spi_write( DDSReg.bytes.b3 );
				spi_write( DDSReg.bytes.b2 );
				spi_write( DDSReg.bytes.b1 );
				SSPBUF = DDSReg.bytes.b0; //direct write to pipeline next code
				uTmpU32.all++;
				if(	uTmpU32.all > (FLASH_VOICEDATA_TABLE+0x17000) )
				{	//if at end of voice data
					uTmpU32.all = FLASH_VOICEDATA_TABLE;
					uTmpU16.bytes.b0 = 0;
				}
				port_c.dds_cs = 1;
				port_a.iosync = 1;
				port_a.iosync = 0;
				port_c.lclr = 0;
			}
			else if( TMR2IF )
			{
				TMR2IF = FALSE;
				if( Mode==MODE_SSTV )
					ServiceSSTV();
				else if( Mode==MODE_FSK441 )
					ServiceFSK441();
			}
			else
			{
				if(	GeneralTmrIRQ && ( Mode==MODE_SSTV ) )
				{
					GeneralTmrIRQ = FALSE;
					if(	Mode==MODE_SSTV )
					{
						StartSSTV();
					}
					ReadTemperature();
				}
			}
		}
		else
		{
			if(	(ParseLevel==PARSE_TOP) && !KeyboardEntryModeActive )
			{
				if(	GeneralTmrIRQ )
				{
					GeneralTmrIRQ = FALSE;
					DisplayTime();
				}
			}
			port_b.ook = FALSE;
			port_c.lclr = 0;
		}
		restart_wdt();
	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                    -----------------------------------                    */
/*                   | C h e c k S p e c i a l C h a r s |                   */
/*                    -----------------------------------                    */
/*    Checks for and processes special control characters from terminal.     */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
#inline void CheckSpecialChars()
{
	switch( InChar )
	{
		case 0x02:	 //CNTRL-B jumps to boot code
			disable_interrupts(GLOBAL);
			Bflag0 = BOOTFLAG0;	//set flags so reboot will
			Bflag1 = BOOTFLAG1;	//enter bootloader,mode
			while(TRUE){};		//let watchdog timer reset CPU
			break;
		case 0x1A:	 //toggle Xmit on and off with 'CTRL-Z'
			if(	XmitOn )
			{
				ReadDDS(DDS_CFR1);
				DDSReg.bytes.b1 |= DDS_OFF;
				WriteDDS(DDS_CFR1, 4);
				port_b.ook = FALSE;
				port_c.lclr = 0;
				XmitOn = FALSE;
			}
			else
			{
				ReadDDS(DDS_CFR1);
				DDSReg.bytes.b1 &= ~DDS_OFF;
				WriteDDS(DDS_CFR1, 4);
				XmitOn = TRUE;
				port_c.lclr = 1;
			}
			if(!KeyboardEntryModeActive)
				Prompt();
			break;
		case 0x1B:	//toggle keyboard entry mode with 'ESC' char
			KeyboardEntryModeActive = ~KeyboardEntryModeActive;
			if(KeyboardEntryModeActive)
				CRLF();
			else
				Prompt();
			break;
	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                        -------------                                      */
/*                       | P r o m p t |                                     */
/*                        -------------                                      */
/*    Send prompt to terminal.                                               */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void Prompt()
{
#if SHORT_DISPLAY
	printf("\n\rF=%LU Hz\n\rMode = ", Frequency.all, Mode);
	switch(Mode)
	{
		case MODE_FSK441:
			printf("FSK441");
			break;
		case MODE_SSTV:
			printf("SSTV");
			break;
		case MODE_VOICE:
			printf("FM VOICE");
			break;
	}
	printf("\r"); 
#else
	printf("\n\rF=%LU Hz  M=%d ", Frequency.all, Mode);
	switch(Mode)
	{
		case MODE_FSK441:
			printf("FSK441");
			break;
		case MODE_SSTV:
			printf("SSTV");
			break;
		case MODE_VOICE:
			printf("FM VOICE");
			break;
	}
	printf("\n\r>"); 
#endif
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                        ---------                                          */
/*                       | C R L F |                                         */
/*                        ---------                                          */
/*    Send CR LF to terminal.                                                */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void CRLF()
{
	putc('\n');
	putc('\r');
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                        --------------------                               */
/*                       | O u t p u t H E X  |                              */
/*                        --------------------                               */
/*    Output 2 digit hex to terminal plus the specified delim character.     */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void OutputHEX(char delim)
{
	printf("%2X%c",TmpU8,delim);
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                     -----------------------                               */
/*                    | P a r s e L e v e l 0 |                              */
/*                     -----------------------                               */
/*    Parse top level ascii characters.                                      */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
#inline void ParseLevel0(char ch)
{
	if(KeyboardEntryModeActive)
		return;
	if(isalpha(ch) )
		ch = toupper(ch);	//deal with only upper case
	ParseIndex = 0;
	sBuf[0] = 0;
	switch(ch)
	{
		case 'F':	//Main frequency entry
			ParseLevel = PARSE_GETNUMBER;
			ParamPtr = PARM_FREQ_0;
			break;
		case 'R':	//Reference frequency entry
			ParseLevel = PARSE_GETNUMBER;
			ParamPtr = PARM_REFFREQ;
			break;
		case 'T':	//Time Entry
			ParseLevel = PARSE_TIME;
			break;
		case 'M':
			ParseLevel = PARSE_GETNUMBER;
			ParamPtr = PARM_MODE;
			break;
		case 'G':
			ParseIndex = INTEE_MSG1;
			ParseLevel = PARSE_GETSTRING;
			break;
		case 'S':
			CRLF();
			ParseIndex = INTEE_MSG1;
			do
			{
				TmpU8 = ReadIntEE(ParseIndex++);
				if(TmpU8)
					putc( TmpU8 );
			}while( TmpU8 && (ParseIndex < (INTEE_MSG1+80) ) );
			CRLF();
//			Prompt();
			break;
		case 'H':
		case '?':
			CRLF();
			TmpS16 = FLASH_TABLE1_HELP1;
			do
			{
				ReadFLASHbase(TmpS16++);
				if(uTmpU32.bytes.b0)
					putc( uTmpU32.bytes.b0 );
			}while( uTmpU32.bytes.b0 && (TmpS16 < (FLASH_TABLE1_HELP1+512) ) );
			Prompt();
			break;
		default:
			ParseLevel = PARSE_TOP;
			break;
	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                     -------------------                                   */
/*                    | P a r s e T i m e |                                  */
/*                     -------------------                                   */
/*    Parse Time entries.                                                    */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
#inline void ParseTime(char ch)
{
	ch = toupper(ch);
	switch(ch)
	{
		case 'M':
			ParseLevel = PARSE_GETNUMBER;
			ParamPtr = PARM_MIN;
			break;
		case 'H':
			ParseLevel = PARSE_GETNUMBER;
			ParamPtr = PARM_HR;
			break;
		case 'D':
			ParseLevel = PARSE_GETNUMBER;
			ParamPtr = PARM_DAY;
			break;
		case 'T':
			ParseLevel = PARSE_GETNUMBER;
			ParamPtr = PARM_DATE;
			break;
		case 'O':
			ParseLevel = PARSE_GETNUMBER;
			ParamPtr = PARM_MON;
			break;
		case 'Y':
			ParseLevel = PARSE_GETNUMBER;
			ParamPtr = PARM_YR;
			break;
		case 'L':	//calibrate
			ParseLevel = PARSE_GETNUMBER;
			ParamPtr = PARM_CLKCAL;
			WriteTOD( TOD_CTRL, FT_BIT);
			break;
		default:	//just read and display time
			ParseLevel = PARSE_TOP;
			DisplayTime();
#if !SHORT_DISPLAY
			Prompt();
#endif
			break;
	}
	ParseIndex = 0;
	sBuf[0] = 0;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                     -------------------                                   */
/*                    | G e t N u m b e r |                                  */
/*                     -------------------                                   */
/*    Get number from incoming ASCII chars and place in sBuf.                */
/*    Result is sent to PutParameter routine to route to specified parameter */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
#inline void GetNumber(char ch)
{
	if(isdigit(ch) )
	{
		if(ParseIndex>=MAX_STR)
		{
			ParseLevel = PARSE_TOP;
			Prompt();
		}
		else
		{
			sBuf[ParseIndex++] = ch;
		}
	}
	else
	{
		if(ch=='\b')	//Backspace
		{
			if(ParseIndex>0)
				ParseIndex--;
		}
		else
		{
			ParseLevel = PARSE_TOP;
			sBuf[ParseIndex] = 0;
			if(ch=='\r')	//return
			{
				if(ParseIndex>0)
				{
					uTmpU32.all = atoi32(sBuf);
					TmpU8 = sBuf[0]-'0';	//get BCD bytes
					swap(TmpU8);
					TmpU8 += sBuf[1]-'0';
					printf("\n  %lu",uTmpU32.all);	//echo it
					PutParameter();
				}
				else
				{
					if( ParamPtr == PARM_REFFREQ)
					{
						printf("\rRef=%lu Hz\n\r", RefFreq_2.all/3);
						InChar = 0;
					}
				}
			}
		}
	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                     -------------------                                   */
/*                    | G e t S t r i n g |                                  */
/*                     -------------------                                   */
/*    Get ascii string from incoming ASCII chars storing it into EEROM.      */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
#inline void GetString(char ch)
{
	if(ch=='\r')	//return
	{
		ParseLevel = PARSE_TOP;
		if(ParseIndex>INTEE_MSG1)
			WriteIntEE(ParseIndex, 0);
		Prompt();
		VarDataPtr = 0xFF;
		Cptr=INTEE_MSG1;
	}
	else
	{
		if(ch=='\\')	//Insert Carriage return/line feed
		{
			ch = '\r';
			WriteIntEE(ParseIndex++, ch);
			if(ParseIndex>=(INTEE_MSG1+80) )
				ParseIndex = INTEE_MSG1+79;
			ch = '\n';
		}
		else
		{
			if(ch=='\b')	//Backspace
			{
				if(ParseIndex>INTEE_MSG1)
					ParseIndex--;
				return;
			}
		}
		WriteIntEE(ParseIndex++, ch);
		if(ParseIndex>=(INTEE_MSG1+80) )
			ParseIndex = INTEE_MSG1+79;
	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                     -------------------------                             */
/*                    | P u t P a r a m e t e r |                            */
/*                     -------------------------                             */
/*    Place new value obtained in uTmpU32 into parameter specified by        */
/*       ParamPtr                                                            */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
#inline void PutParameter()
{
	switch(ParamPtr)
	{
		case PARM_FREQ_0:
			Frequency.all = uTmpU32.all;
			TmpU8 = INTEE_FREQ_BASE + (Mode-1)<<2;
			WriteIntEE(TmpU8,Frequency.bytes.b0);
			WriteIntEE(TmpU8+1,Frequency.bytes.b1);
			WriteIntEE(TmpU8+2,Frequency.bytes.b2);
			WriteIntEE(TmpU8+3,Frequency.bytes.b3);
			InitDDS();
			break;
		case PARM_REFFREQ:
			RefFreq_2 = uTmpU32.all*3;
			WriteIntEE(INTEE_REFFREQ, RefFreq_2.bytes.b0);
			WriteIntEE(INTEE_REFFREQ+1, RefFreq_2.bytes.b1);
			WriteIntEE(INTEE_REFFREQ+2, RefFreq_2.bytes.b2);
			WriteIntEE(INTEE_REFFREQ+3, RefFreq_2.bytes.b3);
			InitDDS();
			break;
		case PARM_MIN:
			WriteTOD( TOD_MIN, TmpU8);
			WriteTOD( TOD_SEC, 0);
			break;
		case PARM_HR:
			WriteTOD( TOD_HR, TmpU8);
			break;
		case PARM_DAY:
			WriteTOD( TOD_DAY, TmpU8);
			break;
		case PARM_DATE:
			WriteTOD( TOD_DATE, TmpU8);
			break;
		case PARM_MON:
			WriteTOD( TOD_MON, TmpU8);
			break;
		case PARM_YR:
			WriteTOD( TOD_YR, TmpU8);
			break;
		case PARM_CLKCAL:
			WriteTOD( TOD_CTRL, uTmpU32.bytes.b0);
			break;
		case PARM_MODE:
			Mode = uTmpU32.bytes.b0;
			WriteIntEE(INTEE_CURMODE,Mode);
			while(TRUE){};		//let watchdog timer reset CPU
			break;
	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                     -----------------------                               */
/*                    | D i s p l a y T i m e |                              */
/*                     -----------------------                               */
/*    Display Current Time.                                                  */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void DisplayTime()
{
#if SHORT_DISPLAY
	printf("\n\r20");
	ReadTOD(TOD_YR);
	OutputHEX('/');
	ReadTOD(TOD_MON);
	OutputHEX('/');
	ReadTOD(TOD_DATE);
	OutputHEX(' ');
	CRLF();
	ReadTOD(TOD_HR);
	OutputHEX(':');
	ReadTOD(TOD_MIN);
	OutputHEX(':');
	ReadTOD(TOD_SEC);
	OutputHEX(' ');
	putc('D');
	ReadTOD(TOD_DAY);
	putc((TmpU8&0x0F)+'0');
#else
	printf("\n\r 20");
	ReadTOD(TOD_YR);
	OutputHEX('/');
	ReadTOD(TOD_MON);
	OutputHEX('/');
	ReadTOD(TOD_DATE);
	OutputHEX(' ');
	ReadTOD(TOD_HR);
	OutputHEX(':');
	ReadTOD(TOD_MIN);
	OutputHEX(':');
	ReadTOD(TOD_SEC);
	OutputHEX(' ');
	putc('D');
	ReadTOD(TOD_DAY);
	putc((TmpU8&0x0F)+'0');
#endif
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                     -----------------------                               */
/*                    | G e t N e x t C h a r |                              */
/*                     -----------------------                               */
/*    Gets next character from EEROM text or from keyboard buffer.           */
/*    Returns character or ZERO if there is nothing in buffer to send.       */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
#inline U8BIT GetNextChar()
{
	if(!KeyboardEntryModeActive)
	{
		if(VarDataPtr != 0xFF)		//if variable data string is source
		{
			TmpU8 = VarStr[ VarDataPtr++];
			if(TmpU8==0)			//if at end of string
				VarDataPtr = 0xFF;
		}
		else
		{
			TmpU8 = ReadIntEE(Cptr++);
			if( (TmpU8==0) || (Cptr>(INTEE_MSG1+80)) )
				Cptr=INTEE_MSG1;
			if(TmpU8 =='^' )
			{
				VarDataPtr = 0;
				TmpU8 = VarStr[ VarDataPtr++];
			}
		}
	}
	else
	{
		if( QHead != QTail )		//get next character to xmit
		{	
			TmpU8 = sBuf[QTail++];
			QTail &= 0x1F;
		}
		else
		{
			TmpU8 = 0;
		}
	}
//	do
//	{
//		TmpU8 = rand()&0x7F;
//	}while( (TmpU8 < ' ') || (TmpU8 > 'Z'));
	return TmpU8;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                      ---------------------------                          */
/*                     | S e r v i c e F S K 4 4 1 |                         */
/*                      ---------------------------                          */
/*    Create FSK441 data stream.                                             */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void ServiceFSK441()
{
	if( ++SymCount >= SYM_PER_CHAR)
	{
		TmpU8 = GetNextChar();
		TmpU8 = toupper( TmpU8 );	//make upper case
		if( TmpU8>=' ' && TmpU8<='Z')
			TmpU8 -= ' ';
		else
			TmpU8 = 0;
		SymAcc = FSK441_TABLE[TmpU8];
		SymCount = 0;
		port_c.lclr = bit_test(LEDtimer++,2);
	}
	DDSReg.all = SymbFreqTbl[SymAcc&0x03];
	WriteDDS(DDS_FTW0, 4);
	SymAcc >>= 2;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                       ---------------                                     */
/*                      | C a l c F T W |                                    */
/*                       ---------------                                     */
/*    This routine calculates the DDS register value from the frequency      */
/*     value in uTmpU32 + the offset.                                        */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void CalcFTW(S32BIT offset)
{
	uTmpU32.all = (S32BIT)uTmpU32.all + offset;
	uTmpU32.all <<= 4;
	Div32by32();
	DDSReg.bytes.b3 = TmpU8<<4;
	Div32by32();
	DDSReg.bytes.b3 += TmpU8;

	Div32by32();
	DDSReg.bytes.b2 = TmpU8<<4;
	Div32by32();
	DDSReg.bytes.b2 += TmpU8;

	Div32by32();
	DDSReg.bytes.b1 = TmpU8<<4;
	Div32by32();
	DDSReg.bytes.b1 += TmpU8;

	Div32by32();
	DDSReg.bytes.b0 = TmpU8<<4;
	Div32by32();
	DDSReg.bytes.b0 += TmpU8;
	DDSReg.all >>= 1;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                       -------------------                                 */
/*                      | D i v 3 2 b y 3 2 |                                */
/*                       -------------------                                 */
/*    This routine divides uTmpU32 by RefFreq_2.                             */
/*    The 8 bit quotient is in TmpU8, 16xRemainder in Aarg.                  */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void Div32by32()
{
	TmpU8 = (U8BIT)(uTmpU32.all/RefFreq_2.all);
	uTmpU32.all = uTmpU32.all%RefFreq_2.all;
	uTmpU32.all = uTmpU32.all << 4;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                       ---------------                                     */
/*                      | I n i t S P I |                                    */
/*                       ---------------                                     */
/*    This routine initializes the SPI port.                                 */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void InitSPI()
{
//	setup_spi( SPI_MASTER|SPI_H_TO_L|SPI_CLK_DIV_4 );
//  library setup doesnt implement Lo to Hi clock correctly
	port_a.iosync = 0;
	SSPSTAT  = 0x40;	//0100 0000b
	SSPCON  = 0x20;		//0010 0000b
	set_tris_c(PORTC_DIR);
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                       ---------------                                     */
/*                      | I n i t I 2 C |                                    */
/*                       ---------------                                     */
/*    This routine initializes the I2C port.                                 */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void InitI2C()
{
	SSPCON = 0x28;		//0010 1000b
	SSPCON2 = 0x00;		//0000 0000b
	SSPSTAT = 0x80;		//1000 0000b
	SSPADD = 9;	//100KHz rate
	set_tris_c(PORTC_I2CDIR);	//SDA SCL must be input bits
	port_a.iosync = 1;		//this signal is used to MUX the SCL line
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                       ---------------                                     */
/*                      | W a i t A C K |                                    */
/*                       ---------------                                     */
/*    This routine waits for the ACK, START, or STOP.                        */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void WaitACK()
{
	TmpU8 = 255;
	bit_clear(PIR1, SSPIF);
	while(!bit_test(PIR1, SSPIF) && TmpU8--){}; bit_clear(PIR1, SSPIF);
	bit_clear(SSPCON2, ACKSTAT);
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                       ---------------                                     */
/*                      | I n i t D D S |                                    */
/*                       ---------------                                     */
/*    This routine initializes the DDS chip.                                 */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void InitDDS()
{
	RefFreq_2.bytes.b0 = ReadIntEE(INTEE_REFFREQ);
	RefFreq_2.bytes.b1 = ReadIntEE(INTEE_REFFREQ+1);
	RefFreq_2.bytes.b2 = ReadIntEE(INTEE_REFFREQ+2);
	RefFreq_2.bytes.b3 = ReadIntEE(INTEE_REFFREQ+3);
	TmpU8 = INTEE_FREQ_BASE + (Mode-1)<<2;
	Frequency.bytes.b0 = ReadIntEE(TmpU8);
	Frequency.bytes.b1 = ReadIntEE(TmpU8+1);
	Frequency.bytes.b2 = ReadIntEE(TmpU8+2);
	Frequency.bytes.b3 = ReadIntEE(TmpU8+3);
	QHead = 0;
	QTail = 0;
	VarDataPtr = 0xFF;
	Cptr=INTEE_MSG1;
	port_b.ook = 0;
	port_b.ddsreset = 1;		//DDS reset active high
	port_b.ddsreset = 0;
	//setup CFR1
	DDSReg.bytes.b0 = 0x42;		//0100 0010	sync clock & Comp disabled
	DDSReg.bytes.b1 = 0x02;		//0000 0010	3 wire SPI mode
	DDSReg.bytes.b2 = 0x00;		//0000 0000 // dis linear sweep
	DDSReg.bytes.b3 = 0x02;		//0000 0010  man/OSK RAM disable
	WriteDDS(DDS_CFR1, 4);
	//setup CFR2
	DDSReg.bytes.b0 = 0x34;		//0011 0100
	DDSReg.bytes.b1 = 0x00;		//0000 0000
	DDSReg.bytes.b2 = 0x00;		//0000 0000
	WriteDDS(DDS_CFR2, 3);
	//setup Amplitude scale factor
	DDSReg.bytes.b0 = 0xFF;		//1111 1111
	DDSReg.bytes.b1 = 0x3F;		//0011 1111
	WriteDDS(DDS_ASF, 2);
	//setup Amplitude increment rate
	DDSReg.bytes.b0 = 50;		//50*164uS = 8mS ramp
	WriteDDS(DDS_ARR, 1);
	disable_interrupts(GLOBAL);
	if(Mode==MODE_FSK441)
	{
		uTmpU32.all = Frequency.all;
		CalcFTW(SYM0_FREQ);
		SymbFreqTbl[0] = DDSReg.all;
		uTmpU32.all = Frequency.all;
		CalcFTW(SYM1_FREQ);
		SymbFreqTbl[1] = DDSReg.all;
		uTmpU32.all = Frequency.all;
		CalcFTW(SYM2_FREQ);
		SymbFreqTbl[2] = DDSReg.all;
		uTmpU32.all = Frequency.all;
		CalcFTW(SYM3_FREQ);
		SymbFreqTbl[3] = DDSReg.all;
		setup_timer_2( T2_DIV_BY_1, (CLOCK_FREQ/(56*441))+0, 14);	//setup for 441Hz irq rate
		enable_interrupts(GLOBAL);
	}
	else if( Mode == MODE_SSTV )
	{
		uTmpU32.all = Frequency.all;
		CalcFTW(VIS_1_FREQ);
		SymbFreqTbl[0] = DDSReg.all;
		uTmpU32.all = Frequency.all;
		CalcFTW(SYNC_FREQ);
		SymbFreqTbl[1] = DDSReg.all;
		uTmpU32.all = Frequency.all;
		CalcFTW(VIS_0_FREQ);
		SymbFreqTbl[2] = DDSReg.all;
		uTmpU32.all = Frequency.all;
		CalcFTW(BLACK_FREQ);
		SymbFreqTbl[3] = DDSReg.all;
		uTmpU32.all = Frequency.all;
		CalcFTW(MID_FREQ);
		SymbFreqTbl[4] = DDSReg.all;
		uTmpU32.all = Frequency.all;
		CalcFTW(WHITE_FREQ);
		SymbFreqTbl[5] = DDSReg.all;
		uTmpU32.all = Frequency.all;
		CalcFTW(BLACK_FREQ+40);
		BlackBaseFreq = DDSReg.all;
		setup_timer_2( T2_DIV_BY_1, 219-1, 3);	//setup for 2283 Hz irq rate
		enable_interrupts(GLOBAL);
	}
	else if( Mode == MODE_VOICE )
	{
		uTmpU32.all = Frequency.all;
		CalcFTW(-3000);
		BlackBaseFreq = DDSReg.all;
		uTmpU32.all = FLASH_VOICEDATA_TABLE;
	}
	else
	{
		uTmpU32.all = Frequency.all;
		CalcFTW(0);
		WriteDDS(DDS_FTW0, 4);
		setup_timer_2( T2_DIV_BY_1, (CLOCK_FREQ/(48*500))-1, 12);	//setup for 500Hz irq rate
		enable_interrupts(GLOBAL);
	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                     -------------------------                             */
/*                    | L o a d D D S T a b l e |                            */
/*                     -------------------------                             */
/*    This routine loads the DDS RAM table from the FLASH table with an      */
/*      offset.                                                              */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void LoadDDSTable(U16BIT EEadr, S32BIT offset)
{
U16BIT adr;
	printf(" Loading..\r\n");
	for( adr=0; adr<1024; adr++)
	{
		//set RAM segment to point to just one 32 bit word at a time
		uTmpU16.all = adr<<2;
		DDSReg.bytes.b0 = uTmpU16.bytes.b1;
		DDSReg.bytes.b1 = uTmpU16.bytes.b0;
		uTmpU16.all = adr+1;
		DDSReg.bytes.b1 |= uTmpU16.bytes.b1;
		DDSReg.bytes.b2 = uTmpU16.bytes.b0;
		DDSReg.bytes.b3 = 0;
		DDSReg.bytes.b4 = 0;
		WriteDDS(DDS_RSCW0, 5);
//
		ReadFLASHbase( (EEadr+adr)<<2 );	//get table value from FLASH
		CalcFTW(offset);
		WriteDDS(DDS_RAM, 4);
		port_c.lclr = ~port_c.lclr;	//toggle LED
	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                       -----------------                                   */
/*                      | W r i t e D D S |                                  */
/*                       -----------------                                   */
/*    This routine writes numbytes from DDSReg to the DDS.                   */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void WriteDDS(U8BIT adr, U8BIT numbytes)
{
	port_c.dds_cs = 0;		//DDS chip select active LOW
	spi_write(adr);
	if(numbytes>4)
		spi_write(DDSReg.bytes.b4);
	if(numbytes>3)
		spi_write(DDSReg.bytes.b3);
	if(numbytes>2)
		spi_write(DDSReg.bytes.b2);
	if(numbytes>1)
		spi_write(DDSReg.bytes.b1);
	spi_write(DDSReg.bytes.b0);
	port_c.dds_cs = 1;
	port_a.iosync = 1;
	port_a.iosync = 0;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                       ---------------                                     */
/*                      | R e a d D D S |                                    */
/*                       ---------------                                     */
/*    This routine reads 4 bytes from DDS to uTmpU32.                        */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void ReadDDS(U8BIT adr)
{
	port_c.dds_cs = 0;		//DDS chip select active LOW
	bit_set(adr,7);
	spi_write(adr);
	DDSReg.bytes.b3 = spi_read(0);
	DDSReg.bytes.b2 = spi_read(0);
	DDSReg.bytes.b1 = spi_read(0);
	DDSReg.bytes.b0 = spi_read(0);
	port_c.dds_cs = 1;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                       -------------------                                 */
/*                      | R e a d F l a s h |                                */
/*                       -------------------                                 */
/*    This routine reads 4 bytes from the Flash from adr and plces them      */
/*      into the data structure uTmpU32.                                     */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void ReadFLASHbase(U16BIT adr)
{
	port_c.ee_cs = 0;		//EEPROM chip select active LOW
	spi_write(FLASH_READ_CMD);
	spi_write( 0 );
	spi_write( make8(adr,1) );
	spi_write( make8(adr,0) );
	uTmpU32.bytes.b0 = spi_read(0);
	uTmpU32.bytes.b1 = spi_read(0);
	uTmpU32.bytes.b2 = spi_read(0);
	uTmpU32.bytes.b3 = spi_read(0);
	port_c.ee_cs = 1;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                       -------------------                                 */
/*                      | R e a d I n t E E |                                */
/*                       -------------------                                 */
/*    This routine reads 1 byte from the internal PIC eerom from adr         */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
U8BIT ReadIntEE(U8BIT adr)
{
	return read_eeprom(adr);
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                       ---------------------                               */
/*                      | W r i t e I n t E E |                              */
/*                       ---------------------                               */
/*    This routine writes 1 byte to the internal PIC eerom from adr          */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void WriteIntEE(U8BIT adr, U8BIT data)
{
   write_eeprom(adr,data);
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                       -----------------                                   */
/*                      | W r i t e T O D |                                  */
/*                       -----------------                                   */
/*    This routine writes data byte to adr in the TOD clock chip.            */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void WriteTOD(U8BIT adr, U8BIT data)
{
	InitI2C();
	bit_set(SSPCON2, SEN);
	WaitACK();
	SSPBUF = I2CWRITE_TOD;	//write slave address and R/W
	WaitACK();
	SSPBUF = adr;	//write TOD register address
	WaitACK();
	SSPBUF = data;		//write data at reg address
	WaitACK();
	bit_set(SSPCON2, PEN);
	WaitACK();
	InitSPI();	//make sure I2C port is off
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                       ---------------                                     */
/*                      | R e a d T O D |                                    */
/*                       ---------------                                     */
/*    This routine reads a data byte at adr in the TOD clock chip.           */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void ReadTOD(U8BIT adr)
{
	InitI2C();
//	First write desired address to TOD chip
	bit_set(SSPCON2, SEN);
	WaitACK();
	SSPBUF = I2CWRITE_TOD;	//write slave address and R/W==write
	WaitACK();
	SSPBUF = adr;	//write TOD register address
	WaitACK();
	bit_set(SSPCON2, PEN);
	WaitACK();
//	now read the data located at the address
	bit_set(SSPCON2, SEN);
	WaitACK();
	SSPBUF = I2CREAD_TOD;	//write slave address and R/W==read
	WaitACK();
	bit_set(SSPCON2, RCEN);	//start reading data
	WaitACK();

	bit_set(SSPCON2, ACKDT);	//nack back to slave
	bit_set(SSPCON2, ACKEN);
	WaitACK();
	bit_set(SSPCON2, PEN);
	WaitACK();
	TmpU8 = SSPBUF;		//get data byte
	InitSPI();	//make sure I2C port is off
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                   -------------------------------                         */
/*                  | R e a d T e m p e r a t u r e |                        */
/*                   -------------------------------                         */
/*    This routine reads teh Temperature A/D and converts to degree text.    */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void ReadTemperature()
{
	TmpS16 = read_adc();
//	TmpS16 -= 131;	//-131	   Celsius 
//	TmpS16 <<= 6;	//*64
//	TmpS16 /= 124;	//

	TmpS16 <<= 6;	//*64		Fahrenheit
	TmpS16 += 3997;	//+3997
	TmpS16 /= 223;	//

	if(	TmpS16>0 )
		VarStr[0] = ' ';
	else
		VarStr[0] = '-';

	VarStr[1] = (TmpS16/10) +'0';
	VarStr[2] = (TmpS16%10) +'0';
//	VarStr[3] = 'C';
	VarStr[3] = 'F';
	VarStr[4] = 0;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                       -------------------                                 */
/*                      | S t a r t S S T V |                                */
/*                       -------------------                                 */
/*    This routine starts the transmission of an SSTV image.                 */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void StartSSTV()
{
	if( SSTVState == STATE_STOPPED )
	{
		DDSReg.all = SymbFreqTbl[4];
		WriteDDS(DDS_FTW0, 4);
		SSTVState = STATE_START1;
		SymCount = 200;
		BitCount = 0;
		port_a.xio_0 = 0;
		PR2 = PR2_SYNCRELOAD;
		ImagePointer.all = FLASH_SSTVDATA_TABLE;
	}
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*                       -----------------------                             */
/*                      | S e r v i c e S S T V |                            */
/*                       -----------------------                             */
/*    This routine generates the SSTV data stream using a state machine for  */
/* all the various parts of the SSTV signal.                                 */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void ServiceSSTV()
{
	if( SSTVState == STATE_STOPPED )
	{
port_c.lclr = 0;
		return;
	}
	switch(SSTVState)
	{
		case STATE_START1:	//create 100msec "diddles"
			if(--SymCount == 0 )
			{
				SymCount = 200;
				if(++BitCount >= 8 )
				{
					BitCount = 0;
					SSTVState = STATE_START2;
					DDSReg.all = SymbFreqTbl[4];
				}
				if( bit_test(BitCount,0) )
				{
					DDSReg.all = SymbFreqTbl[3];
				}
				else
				{
					if( BitCount > 3 )
						DDSReg.all = SymbFreqTbl[5];
					else
						DDSReg.all = SymbFreqTbl[4];
				}
				WriteDDS(DDS_FTW0, 4);
			}
			break;
		case STATE_START2:	//create 300msec "diddles"
			if(--SymCount == 0 )
			{
				if(++BitCount >= 7 )
				{
					SymAcc = VIS_S1;
					SymCount = SYMBPERVIS;
					BitCount = 0;
					SSTVState = STATE_VIS;
					DDSReg.all = SymbFreqTbl[1];
				}
				else
				{
					if( BitCount == 3 )
					{
						DDSReg.all = SymbFreqTbl[3];
						SymCount = 20;
					}
					else
					{
						DDSReg.all = SymbFreqTbl[4];
						SymCount = 200;
					}
				}
				WriteDDS(DDS_FTW0, 4);
			}
			break;
		case STATE_VIS:
			if(--SymCount == 0 )
			{
				SymCount = SYMBPERVIS;
				if(++BitCount >= 9 )
				{
					LineCount = NUMLINES;
					SymCount += SYMBPERSYNC;
					SSTVState = STATE_FIRSTSYNC;
					DDSReg.all = SymbFreqTbl[1];
				}
				else
				{
					if(BitCount <8 )
					{
						if( bit_test(SymAcc,0) )
							DDSReg.all = SymbFreqTbl[0];
						else
							DDSReg.all = SymbFreqTbl[2];
						SymAcc >>= 1;
					}
					else
					{
						DDSReg.all = SymbFreqTbl[2];	//parity
					}
				}
				WriteDDS(DDS_FTW0, 4);
			}
			break;
		case STATE_FIRSTSYNC:
			if(--SymCount == 0 )
			{
				if(	LineCount-- == 0)
				{
					SSTVState = STATE_STOPPED;
				}
				else
				{
					PixCount = PIXPERLINE;
					SSTVState = STATE_GREEN;
					PR2 = PR2_PIXRELOAD;
				}
			}
			break;
		case STATE_SYNC:
			if(--SymCount == 0 )
			{
				if(	LineCount-- == 0)
				{
					SSTVState = STATE_STOPPED;
				}
				else
				{
					port_c.lclr = 0;
					PixCount = PIXPERLINE;
					SSTVState = STATE_RED;
					PR2 = PR2_PIXRELOAD;
				}
			}
			break;
		case STATE_RED:
			if(	PixCount-- == 0)
			{
				PixCount = PIXPERLINE;
				SSTVState = STATE_GREEN;
			}
			else
			{
				SetPixelFreq();
			}
			break;
		case STATE_GREEN:
			if(	PixCount-- == 0)
			{
				PixCount = PIXPERLINE;
				SSTVState = STATE_BLUE;
			}
			else
			{
				SetPixelFreq();
			}
			break;
		case STATE_BLUE:
			if(	PixCount-- == 0)
			{
				port_c.lclr = 1;
				SymCount = SYMBPERSYNC;
				SSTVState = STATE_SYNC;
				DDSReg.all = SymbFreqTbl[1];
				WriteDDS(DDS_FTW0, 4);
				PR2 = PR2_SYNCRELOAD;
			}
			else
			{
				SetPixelFreq();
			}
			break;
		case STATE_STOPPED:
			port_c.lclr = 0;
			break;
	}
}

void SetPixelFreq()
{
	port_c.ee_cs = 0;		//EEPROM chip select active LOW
	spi_write(FLASH_READ_CMD);
	spi_write( ImagePointer.bytes.b2 );
	spi_write( ImagePointer.bytes.b1 );
	spi_write( ImagePointer.bytes.b0 );
	uTmpU32.bytes.b0 = spi_read(0);
	port_c.ee_cs = 1;
	uTmpU32.bytes.b1 = 0;
	uTmpU32.bytes.b2 = 0;
	uTmpU32.bytes.b3 = 0;
	uTmpU32.all <<= 5;
	DDSReg.all = BlackBaseFreq + uTmpU32.all;
	port_c.dds_cs = 0;		//DDS chip select active LOW
	spi_write(DDS_FTW0);
	spi_write( DDSReg.bytes.b3 );
	spi_write( DDSReg.bytes.b2 );
	spi_write( DDSReg.bytes.b1 );
	spi_write( DDSReg.bytes.b0 );
	port_c.dds_cs = 1;
	port_a.iosync = 1;
	port_a.iosync = 0;
	ImagePointer.all++;
}
